;; Generated package description from nano-modeline.el  -*- no-byte-compile: t -*-
(define-package "nano-modeline" "0.3" "N Λ N O modeline" '((emacs "27.1")) :maintainer '("Nicolas P. Rougier" . "Nicolas.Rougier@inria.fr") :keywords '("convenience" "mode-line" "header-line") :url "https://github.com/rougier/nano-modeline")
